@@ -155,7 +155,7 @@ class ActionConfigFile(Action, FilesCompleterMethod):
     def __init__(self, **kwargs):
         """Initializer for ActionConfigFile instance."""
         if 'default' in kwargs:
-            raise ValueError('default not allowed for ActionConfigFile, use default_config_files.')
+            self.set_default_error()
         opt_name = kwargs['option_strings']
         opt_name = opt_name[0] if len(opt_name) == 1 else [x for x in opt_name if x[0:2] == '--'][0]
         if '.' in opt_name:
@@ -172,11 +172,14 @@ class ActionConfigFile(Action, FilesCompleterMethod):
         """
         self.apply_config(parser, cfg, self.dest, values)
 
+    @staticmethod
+    def set_default_error():
+        raise ValueError('ActionConfigFile does not accept a default, use default_config_files.')
+
     @staticmethod
     def apply_config(parser, cfg, dest, value) -> None:
-        with _ActionSubCommands.not_single_subcommand(), previous_config_context(cfg):
-            if cfg.get(dest) is None:
-                cfg[dest] = []
+        from .link_arguments import skip_apply_links
+        with _ActionSubCommands.not_single_subcommand(), previous_config_context(cfg), skip_apply_links():
             kwargs = {'env': False, 'defaults': False, '_skip_check': True, '_fail_no_subcommand': False}
             try:
                 cfg_path: Optional[Path] = Path(value, mode=get_config_read_mode())
@@ -190,10 +193,11 @@ class ActionConfigFile(Action, FilesCompleterMethod):
                     raise TypeError(f'Parser key "{dest}": {ex_str}') from ex_str
             else:
                 cfg_file = parser.parse_path(value, **kwargs)
+            cfg_merged = parser.merge_config(cfg_file, cfg)
+            cfg.__dict__.update(cfg_merged.__dict__)
+            if cfg.get(dest) is None:
+                cfg[dest] = []
             cfg[dest].append(cfg_path)
-            cfg.update(cfg_file)
-            from .typehints import ActionTypeHint
-            ActionTypeHint.apply_appends(parser, cfg)
 
 
 previous_config: ContextVar = ContextVar('previous_config', default=None)
@@ -322,7 +326,11 @@ class _ActionHelpClassPath(Action):
 
     def update_init_kwargs(self, kwargs):
         if getattr(self._baseclass, '__origin__', None) == Union:
-            self._basename = iter_to_set_str(c.__name__ for c in self._baseclass.__args__ if c != NoneType)
+            from .typehints import ActionTypeHint
+            self._basename = iter_to_set_str(
+                c.__name__ for c in self._baseclass.__args__
+                if ActionTypeHint.is_subclass_typehint(c)
+            )
         else:
             self._basename = self._baseclass.__name__
         kwargs.update({
@@ -340,27 +348,28 @@ class _ActionHelpClassPath(Action):
 
     def print_help(self, call_args, baseclass, dest):
         from .typehints import resolve_class_path_by_name
+        parser, _, value, option_string = call_args
         try:
-            val_class = import_object(resolve_class_path_by_name(baseclass, call_args[2]))
+            val_class = import_object(resolve_class_path_by_name(baseclass, value))
         except Exception as ex:
-            raise TypeError(f'{call_args[3]}: {ex}')
+            raise TypeError(f'{option_string}: {ex}')
         if getattr(self._baseclass, '__origin__', None) == Union:
             baseclasses = self._baseclass.__args__
         else:
             baseclasses = [baseclass]
         if not any(is_subclass(val_class, b) for b in baseclasses):
-            raise TypeError(f'{call_args[3]}: Class "{call_args[2]}" is not a subclass of {self._basename}')
+            raise TypeError(f'{option_string}: Class "{value}" is not a subclass of {self._basename}')
         dest += '.init_args'
-        subparser = import_object('jsonargparse.ArgumentParser')()
+        subparser = type(parser)()
         subparser.add_class_arguments(val_class, dest, **self.sub_add_kwargs)
         _remove_actions(subparser, (_HelpAction, _ActionPrintConfig, _ActionConfigLoad))
-        args = self.get_args_after_opt(call_args[0].args)
+        args = self.get_args_after_opt(parser.args)
         if args:
             subparser.parse_args(args)
             raise ParserError(f'Expected a nested --*.help option, got: {args}.')
         else:
             subparser.print_help()
-            call_args[0].exit()
+            parser.exit()
 
     def get_args_after_opt(self, args):
         opt_str = self.option_strings[0]
@@ -567,6 +576,7 @@ class _ActionSubCommands(_SubParsersAction):
 
         parser.prog = f'{self._prog_prefix} [options] {name}'
         parser.env_prefix = f'{self._env_prefix}_{name}_'
+        parser.default_env = self.parent_parser.default_env
         parser.parent_parser = self.parent_parser
         parser.parser_mode = self.parent_parser.parser_mode
         parser.subcommand = name
