@@ -11,11 +11,20 @@ from copy import deepcopy
 from typing import Any, Callable, Dict, List, NoReturn, Optional, Sequence, Set, Tuple, Type, Union
 
 from .formatters import DefaultHelpFormatter, empty_help, formatter_context, get_env_var
-from .jsonnet import ActionJsonnet
+from .jsonnet import ActionJsonnet, ActionJsonnetExtVars
 from .jsonschema import ActionJsonSchema
 from .link_arguments import ActionLink, ArgumentLinking
-from .loaders_dumpers import check_valid_dump_format, dump_using_format, get_loader_exceptions, loaders, load_value, load_value_context, yaml_load
-from .namespace import dict_to_namespace, is_meta_key, Namespace, patch_namespace, split_key, split_key_leaf, strip_meta
+from .loaders_dumpers import (
+    check_valid_dump_format,
+    dump_using_format,
+    get_loader_exceptions,
+    loaders,
+    load_value,
+    load_value_context,
+    set_omegaconf_loader,
+    yaml_load,
+)
+from .namespace import is_meta_key, Namespace, patch_namespace, split_key, split_key_leaf, strip_meta
 from .signatures import is_pure_dataclass, SignatureArguments
 from .typehints import ActionTypeHint, is_subclass_spec
 from .typing import is_final_class
@@ -42,12 +51,12 @@ from .optionals import (
     get_config_read_mode,
     import_fsspec,
     import_jsonnet,
-    omegaconf_support,
 )
 from .util import (
     change_to_path_dir,
     get_private_kwargs,
     identity,
+    is_subclass,
     lenient_check,
     lenient_check_context,
     ParserError,
@@ -83,10 +92,13 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
             enable_path: Whether to try parsing path/subconfig when argument is a complex type.
         """
         parser = self.parser if hasattr(self, 'parser') else self  # type: ignore
-        if 'action' in kwargs and isinstance(kwargs['action'], ActionParser):
-            if kwargs['action']._parser == parser:
-                raise ValueError('Parser cannot be added as a subparser of itself.')
-            return ActionParser._move_parser_actions(parser, args, kwargs)
+        if 'action' in kwargs:
+            if isinstance(kwargs['action'], ActionParser):
+                if kwargs['action']._parser == parser:
+                    raise ValueError('Parser cannot be added as a subparser of itself.')
+                return ActionParser._move_parser_actions(parser, args, kwargs)
+            if is_subclass(kwargs['action'], ActionConfigFile) and any(isinstance(a, ActionConfigFile) for a in self._actions):
+                raise ValueError('A parser is only allowed to have a single ActionConfigFile argument.')
         if 'type' in kwargs:
             if is_final_class(kwargs['type']) or is_pure_dataclass(kwargs['type']):
                 theclass = kwargs.pop('type')
@@ -103,6 +115,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
                     kwargs=kwargs,
                     enable_path=enable_path,
                     container=super(),
+                    logger=self.logger,
                 )
         action = super().add_argument(*args, **kwargs)
         if isinstance(action, ActionConfigFile) and getattr(self, '_print_config', None) is not None:
@@ -215,7 +228,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
     def parse_known_args(self, args=None, namespace=None):
         """Raises NotImplementedError to dissuade its use, since typos in configs would go unnoticed."""
-        caller = inspect.getmodule(inspect.stack()[1][0]).__package__
+        caller_mod = inspect.getmodule(inspect.stack()[1][0])
+        caller = None if caller_mod is None else caller_mod.__package__
         if caller not in {'jsonargparse', 'argcomplete'}:
             raise NotImplementedError('parse_known_args not implemented to dissuade its use, since typos in configs would go unnoticed.')
 
@@ -293,10 +307,10 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
         _ActionPrintConfig.print_config_if_requested(self, cfg)
 
-        ActionLink.apply_parsing_links(self, cfg)
+        with load_value_context(self.parser_mode):
+            ActionLink.apply_parsing_links(self, cfg)
 
-        if not skip_check and not lenient_check.get():
-            with load_value_context(self.parser_mode):
+            if not skip_check and not lenient_check.get():
                 self.check_config(cfg, skip_required=skip_required)
 
         if log_message is not None:
@@ -410,7 +424,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             if cfg_base:
                 cfg = self.merge_config(cfg_base, cfg)
 
-            cfg_obj = self._apply_actions(cfg_obj)
+            cfg_obj = self._apply_actions(cfg_obj, prev_cfg=cfg)
             cfg = self.merge_config(cfg_obj, cfg)
 
             parsed_cfg = self._parse_common(
@@ -447,7 +461,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                         cfg[subcommand+'.'+k] = v
         for action in actions:
             env_var = get_env_var(self, action)
-            if env_var in env and not isinstance(action, ActionConfigFile):
+            if env_var in env and not isinstance(action, (ActionConfigFile, _ActionSubCommands)):
                 env_val = env[env_var]
                 if _is_action_value_list(action):
                     if re.match('^ *\\[.+,.+] *$', env_val):
@@ -856,11 +870,13 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             KeyError: If key not defined in the parser.
         """
         if len(args) > 0:
-            for n in range(len(args)):
-                for dest, default in args[n].items():
+            for arg in args:
+                for dest, default in arg.items():
                     action = _find_action(self, dest)
                     if action is None:
                         raise KeyError(f'No action for destination key "{dest}" to set its default.')
+                    elif isinstance(action, ActionConfigFile):
+                        ActionConfigFile.set_default_error()
                     action.default = default
                     if isinstance(action, ActionTypeHint):
                         action.normalize_default()
@@ -1039,6 +1055,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                     except TypeError as ex:
                         if not (val == {} and ActionTypeHint.is_subclass_typehint(action) and key not in self.required_args):
                             raise ex
+                elif key in self.groups and hasattr(self.groups[key], 'instantiate_class'):
+                    raise TypeError(f'Class group {key!r} got an unexpected value: {val}.')
                 else:
                     raise KeyError(f'No action for destination key "{key}" to check its value.')
 
@@ -1192,17 +1210,25 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             keys = list(cfg.__dict__.keys())
 
         if prev_cfg:
-            prev_cfg = prev_cfg.clone().update(dict_to_namespace(cfg))
+            prev_cfg = prev_cfg.clone()
         else:
-            prev_cfg = cfg
+            prev_cfg = Namespace()
 
         config_keys: Set[str] = set()
         num = 0
         while num < len(keys):
             key = keys[num]
-            num += 1
             exclude = _ActionConfigLoad if key in config_keys else None
             action, subcommand = _find_action_and_subcommand(self, key, exclude=exclude)
+
+            if isinstance(action, ActionJsonnet):
+                ext_vars_key = action._ext_vars
+                if ext_vars_key and ext_vars_key not in keys[:num]:
+                    keys = keys[:num] + [ext_vars_key] + [k for k in keys[num:] if k != ext_vars_key]
+                    continue
+
+            num += 1
+
             if action is None or isinstance(action, _ActionSubCommands):
                 value = cfg[key]
                 if isinstance(value, dict):
@@ -1212,6 +1238,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                     keys += [key+'.'+k for k in new_keys if key+'.'+k not in keys]
                 cfg[key] = value
                 continue
+
             action_dest = action.dest if subcommand is None else subcommand+'.'+action.dest
             value = cfg[action_dest]
             with lenient_check_context(), load_value_context(self.parser_mode):
@@ -1219,6 +1246,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             if isinstance(action, _ActionConfigLoad):
                 config_keys.add(action_dest)
                 keys.append(action_dest)
+            elif isinstance(action, ActionJsonnetExtVars):
+                prev_cfg[action_dest] = value
             cfg[action_dest] = value
         return cfg[parent_key] if parent_key else cfg
 
@@ -1240,7 +1269,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         return cfg
 
 
-    def _check_value_key(self, action: argparse.Action, value: Any, key: str, cfg: Namespace) -> Any:
+    def _check_value_key(self, action: argparse.Action, value: Any, key: str, cfg: Optional[Namespace]) -> Any:
         """Checks the value for a given action.
 
         Args:
@@ -1261,13 +1290,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             subparser.check_config(value)
         elif isinstance(action, _ActionConfigLoad):
             if isinstance(value, str):
-                fpath = None
-                if '.' in key:
-                    parent = cfg.get(split_key_leaf(key)[0])
-                    if isinstance(parent, Namespace):
-                        fpath = parent.get('__path__')
-                with change_to_path_dir(fpath):
-                    value = action.check_type(value, self)
+                value = action.check_type(value, self)
         elif hasattr(action, '_check_type'):
             value = action._check_type(value, cfg=cfg)  # type: ignore
         elif action.type is not None:
@@ -1428,10 +1451,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
     @parser_mode.setter
     def parser_mode(self, parser_mode: str):
-        if parser_mode not in loaders and parser_mode == 'omegaconf' and omegaconf_support:
-            from .loaders_dumpers import set_loader
-            from .optionals import get_omegaconf_loader
-            set_loader('omegaconf', get_omegaconf_loader())
+        if parser_mode == 'omegaconf':
+            set_omegaconf_loader()
         if parser_mode not in loaders:
             raise ValueError(f'The only accepted values for parser_mode are {set(loaders.keys())}.')
         if parser_mode == 'jsonnet':
