@@ -1,23 +1,18 @@
 """Action to support jsonschemas."""
 
-import json
 import os
-import yaml
 from argparse import Action
 from typing import Dict, Union
 
 from .actions import _is_action_value_list
+from .loaders_dumpers import get_loader_exceptions, load_value, load_value_context
 from .namespace import strip_meta
 from .optionals import (
     argcomplete_warn_redraw_prompt,
+    get_jsonschema_exceptions,
     import_jsonschema,
-    jsonschemaValidationError,
-)
-from .util import (
-    _parse_value_or_config,
-    yamlParserError,
-    yamlScannerError,
 )
+from .util import parse_value_or_config
 
 
 __all__ = ['ActionJsonSchema']
@@ -46,10 +41,11 @@ class ActionJsonSchema(Action):
         """
         if schema is not None:
             if isinstance(schema, str):
-                try:
-                    schema = yaml.safe_load(schema)
-                except (yamlParserError, yamlScannerError) as ex:
-                    raise ValueError('Problems parsing schema :: '+str(ex)) from ex
+                with load_value_context('yaml'):
+                    try:
+                        schema = load_value(schema)
+                    except get_loader_exceptions() as ex:
+                        raise ValueError(f'Problems parsing schema :: {ex}') from ex
             jsonvalidator = import_jsonschema('ActionJsonSchema')[1]
             jsonvalidator.check_schema(schema)
             self._validator = self._extend_jsonvalidator_with_default(jsonvalidator)(schema)
@@ -75,8 +71,10 @@ class ActionJsonSchema(Action):
             kwargs['_enable_path'] = self._enable_path
             kwargs['_with_meta'] = self._with_meta
             if 'help' in kwargs and isinstance(kwargs['help'], str) and '%s' in kwargs['help']:
+                import json
                 kwargs['help'] = kwargs['help'] % json.dumps(self._validator.schema, sort_keys=True)
-            return ActionJsonSchema(**kwargs)
+            class_type = kwargs.pop('_class_type', ActionJsonSchema)
+            return class_type(**kwargs)
         val = self._check_type(args[2])
         if not self._with_meta:
             val = strip_meta(val)
@@ -89,7 +87,7 @@ class ActionJsonSchema(Action):
             value = [value]
         for num, val in enumerate(value):
             try:
-                val, fpath = _parse_value_or_config(val, enable_path=self._enable_path)
+                val, fpath = parse_value_or_config(val, enable_path=self._enable_path)
                 path_meta = val.pop('__path__') if isinstance(val, dict) and '__path__' in val else None
                 self._validator.validate(val)
                 if path_meta is not None:
@@ -97,7 +95,7 @@ class ActionJsonSchema(Action):
                 if isinstance(val, dict) and fpath is not None:
                     val['__path__'] = fpath
                 value[num] = val
-            except (TypeError, ValueError, yamlParserError, yamlScannerError, jsonschemaValidationError) as ex:
+            except (TypeError, ValueError) + get_jsonschema_exceptions() + get_loader_exceptions() as ex:
                 elem = '' if not islist else ' element '+str(num+1)
                 raise TypeError(f'Parser key "{self.dest}"{elem}: {ex}') from ex
         return value if islist else value[0]
@@ -113,8 +111,7 @@ class ActionJsonSchema(Action):
                 if 'default' in subschema:
                     instance.setdefault(prop, subschema['default'])
 
-            for error in validate_properties(validator, properties, instance, schema):
-                yield error
+            yield from validate_properties(validator, properties, instance, schema)
 
         jsonschema = import_jsonschema('ActionJsonSchema')[0]
         return jsonschema.validators.extend(validator_class, {'properties': set_defaults})
@@ -126,11 +123,12 @@ class ActionJsonSchema(Action):
             try:
                 if prefix.strip() == '':
                     raise ValueError()
-                self._validator.validate(yaml.safe_load(prefix))
+                self._validator.validate(load_value(prefix))
                 msg = 'value already valid, '
-            except (ValueError, yamlParserError, yamlScannerError, jsonschemaValidationError):
+            except (ValueError,) + get_jsonschema_exceptions() + get_loader_exceptions():
                 msg = 'value not yet valid, '
             else:
+                import json
                 schema = json.dumps(self._validator.schema, indent=2, sort_keys=True).replace('\n', '\n  ')
                 msg += f'required to be valid according to schema:\n  {schema}\n'
             return argcomplete_warn_redraw_prompt(prefix, msg)
