@@ -1,14 +1,13 @@
 """Code related to loading and dumping."""
 
 import inspect
-import json
 import re
 import yaml
 from contextlib import contextmanager
 from contextvars import ContextVar
 from typing import Any, Callable, Dict, Tuple, Type
 
-from .optionals import dump_preserve_order_support, import_jsonnet
+from .optionals import import_jsonnet, omegaconf_support
 from .type_checking import ArgumentParser
 
 
@@ -107,23 +106,26 @@ def get_loader_exceptions():
     return loader_exceptions[load_value_mode.get()]
 
 
-def load_value(value: str, **kwargs):
+def load_value(value: str, simple_types: bool = False, **kwargs):
     loader = loaders[load_value_mode.get()]
     if kwargs:
         params = set(list(inspect.signature(loader).parameters)[1:])
         kwargs = {k: v for k, v in kwargs.items() if k in params}
-    return loader(value, **kwargs)
+    loaded_value = loader(value, **kwargs)
+    if not simple_types and isinstance(loaded_value, (int, float, bool)):
+        loaded_value = value
+    return loaded_value
 
 
 dump_yaml_kwargs = {
     'default_flow_style': False,
     'allow_unicode': True,
-    'sort_keys': False if dump_preserve_order_support else True,
+    'sort_keys': False,
 }
 
 dump_json_kwargs = {
     'ensure_ascii': False,
-    'sort_keys': False if dump_preserve_order_support else True,
+    'sort_keys': False,
 }
 
 
@@ -138,10 +140,12 @@ def yaml_comments_dump(data, parser):
 
 
 def json_dump(data):
+    import json
     return json.dumps(data, separators=(',', ':'), **dump_json_kwargs)
 
 
 def json_indented_dump(data):
+    import json
     return json.dumps(data, indent=2, **dump_json_kwargs)+'\n'
 
 
@@ -203,3 +207,9 @@ def set_dumper(format_name: str, dumper_fn: Callable[[Any], str]):
         dumper_fn: The dumper function to set. Example: ``yaml.safe_dump``.
     """
     dumpers[format_name] = dumper_fn
+
+
+def set_omegaconf_loader():
+    if omegaconf_support and 'omegaconf' not in loaders:
+        from .optionals import get_omegaconf_loader
+        set_loader('omegaconf', get_omegaconf_loader())
