@@ -3,7 +3,6 @@
 import inspect
 import locale
 import os
-import platform
 import typing
 from contextlib import contextmanager
 from importlib.util import find_spec
@@ -27,7 +26,6 @@ fsspec_support = find_spec('fsspec') is not None
 ruyaml_support = find_spec('ruyaml') is not None
 omegaconf_support = find_spec('omegaconf') is not None
 reconplogger_support = find_spec('reconplogger') is not None
-dump_preserve_order_support = platform.python_implementation() == 'CPython'
 
 _config_read_mode = 'fr'
 _docstring_parse_options = {
@@ -44,7 +42,7 @@ def typing_extensions_import(name):
 
 
 final = typing_extensions_import('final')
-if not getattr(final(Namespace()), '__final__', False):
+if not final or not getattr(final(Namespace()), '__final__', False):
     def final(cls):  # pylint: disable=function-redefined
         """Decorator to make a class ``final``, i.e. it shouldn't be subclassed."""
         setattr(cls, '__final__', True)
