@@ -7,6 +7,7 @@ from contextlib import contextmanager
 from contextvars import ContextVar
 from copy import deepcopy
 from dataclasses import dataclass
+from functools import partial
 from typing import Any, Callable, List, Optional, Tuple, Type, Union
 
 from .util import ClassFromFunctionBase, is_subclass, LoggerProperty, parse_logger
@@ -218,7 +219,7 @@ def get_signature_parameters_and_indexes(component, parent, logger):
 
 
 ast_literals = {
-    ast.dump(ast.parse(v, mode='eval').body): lambda: ast.literal_eval(v)
+    ast.dump(ast.parse(v, mode='eval').body): partial(ast.literal_eval, v)
     for v in ['{}', '[]']
 }
 
@@ -256,7 +257,7 @@ def replace_args_and_kwargs(params: ParamList, args: ParamList, kwargs: ParamLis
     kwargs_idx = get_arg_kind_index(params, kinds.VAR_KEYWORD)
     if args_idx >= 0:
         params = params[:args_idx] + args + params[args_idx+1:]
-        if kwargs_idx != 1:
+        if kwargs_idx >= 0:
             kwargs_idx += len(args) - 1
     if kwargs_idx >= 0:
         existing_names = set(p.name for p in params[:kwargs_idx] + params[kwargs_idx+1:])
