@@ -32,9 +32,9 @@ from typing import (
 )
 
 from .actions import _ActionHelpClassPath, _find_action, _find_parent_action, _is_action_value_list
-from .loaders_dumpers import get_loader_exceptions, load_value, load_value_context
+from .loaders_dumpers import get_loader_exceptions, load_value, load_value_context, pyyaml_exceptions, yaml_load
 from .namespace import Namespace
-from .typing import is_final_class, registered_types
+from .typing import get_registered_type, is_final_class
 from .optionals import (
     argcomplete_warn_redraw_prompt,
     get_files_completer,
@@ -48,7 +48,6 @@ from .util import (
     indent_text,
     is_subclass,
     iter_to_set_str,
-    lenient_check,
     lenient_check_context,
     NestedArg,
     NoneType,
@@ -124,6 +123,19 @@ class ActionTypeHint(Action):
         if typehint is not None:
             if not self.is_supported_typehint(typehint, full=True):
                 raise ValueError(f'Unsupported type hint {typehint}.')
+            if get_typehint_origin(typehint) == Union:
+                subtype_supported = [
+                    subtype is NoneType or self.is_supported_typehint(subtype, full=True)
+                    for subtype in typehint.__args__
+                ]
+                if sum(subtype_supported) < len(subtype_supported):
+                    discard = {typehint.__args__[n] for n, s in enumerate(subtype_supported) if not s}
+                    kwargs['logger'].debug(f'Discarding unsupported subtypes {discard} from {typehint}')
+                    orig_typehint = typehint  # deepcopy does not copy ForwardRef
+                    typehint = deepcopy(orig_typehint)
+                    typehint.__args__ = tuple(
+                        orig_typehint.__args__[n] for n, s in enumerate(subtype_supported) if s
+                    )
             self._typehint = typehint
             self._enable_path = False if is_optional(typehint, Path) else enable_path
         elif '_typehint' not in kwargs:
@@ -144,6 +156,7 @@ class ActionTypeHint(Action):
             self.default = default.lazy_get_init_data()
         elif self.is_subclass_typehint(self._typehint, all_subtypes=False) and isinstance(default, dict) and 'class_path' in default:
             self.default = subclass_spec_as_namespace(default)
+            self.default.class_path = normalize_import_path(self.default.class_path, self._typehint)
         elif is_enum_type(self._typehint) and isinstance(default, Enum):
             self.default = default.name
         elif is_callable_type(self._typehint) and callable(default) and not inspect.isclass(default):
@@ -151,17 +164,18 @@ class ActionTypeHint(Action):
 
 
     @staticmethod
-    def prepare_add_argument(args, kwargs, enable_path, container, sub_add_kwargs=None):
+    def prepare_add_argument(args, kwargs, enable_path, container, logger, sub_add_kwargs=None):
         if 'action' in kwargs:
             raise ValueError('Providing both type and action not allowed.')
         typehint = kwargs.pop('type')
         if args[0].startswith('--') and ActionTypeHint.supports_append(typehint):
             args = tuple(list(args)+[args[0]+'+'])
-        if ActionTypeHint.is_subclass_typehint(typehint):
-            help_action = container.add_argument(args[0]+'.help', action=_ActionHelpClassPath(baseclass=typehint))
+        if ActionTypeHint.is_subclass_typehint(typehint, all_subtypes=False):
+            help_option = f'--{args[0]}.help' if args[0][0] != '-' else f'{args[0]}.help'
+            help_action = container.add_argument(help_option, action=_ActionHelpClassPath(baseclass=typehint))
             if sub_add_kwargs:
                 help_action.sub_add_kwargs = sub_add_kwargs
-        kwargs['action'] = ActionTypeHint(typehint=typehint, enable_path=enable_path)
+        kwargs['action'] = ActionTypeHint(typehint=typehint, enable_path=enable_path, logger=logger)
         return args
 
 
@@ -171,20 +185,27 @@ class ActionTypeHint(Action):
         supported = \
             typehint in root_types or \
             get_typehint_origin(typehint) in root_types or \
-            typehint in registered_types or \
+            get_registered_type(typehint) is not None or \
             is_subclass(typehint, Enum) or \
             ActionTypeHint.is_subclass_typehint(typehint)
         if full and supported:
             typehint_origin = get_typehint_origin(typehint) or typehint
             if typehint not in root_types and typehint_origin in root_types and typehint_origin != Literal:
-                for typehint in getattr(typehint, '__args__', []):
-                    if not (
-                        typehint == Ellipsis or
-                        (typehint_origin == type and isinstance(typehint, TypeVar)) or
-                        typehint in leaf_types or
-                        ActionTypeHint.is_supported_typehint(typehint, full=True)
+                num_supported_args = 0
+                subtypes = getattr(typehint, '__args__', [])
+                subtypes = [s for s in subtypes if s is not NoneType]
+                for subtype in subtypes:
+                    if (
+                        subtype == Ellipsis or
+                        (typehint_origin == type and isinstance(subtype, TypeVar)) or
+                        subtype in leaf_types or
+                        ActionTypeHint.is_supported_typehint(subtype, full=True)
                     ):
+                        num_supported_args += 1
+                    elif typehint_origin != Union:
                         return False
+                if typehint_origin == Union and subtypes and num_supported_args == 0:
+                    return False
         return supported
 
 
@@ -200,7 +221,7 @@ class ActionTypeHint(Action):
             return test(ActionTypeHint.is_subclass_typehint(s) for s in subtypes)
         return inspect.isclass(typehint) and \
             typehint not in leaf_or_root_types and \
-            typehint not in registered_types and \
+            not get_registered_type(typehint) and \
             typehint_origin is None and \
             not is_subclass(typehint, (Path, Enum))
 
@@ -213,6 +234,16 @@ class ActionTypeHint(Action):
         return False
 
 
+    @staticmethod
+    def is_callable_typehint(typehint, all_subtypes=True):
+        typehint_origin = get_typehint_origin(typehint)
+        if typehint_origin == Union:
+            subtypes = [a for a in typehint.__args__ if a != NoneType]
+            test = all if all_subtypes else any
+            return test(ActionTypeHint.is_callable_typehint(s) for s in subtypes)
+        return typehint_origin in callable_origin_types or typehint in callable_origin_types
+
+
     def is_init_arg_mapping_typehint(self, key, cfg):
         result = False
         class_path = cfg.get(f'{self.dest}.class_path')
@@ -238,8 +269,9 @@ class ActionTypeHint(Action):
                 action = _find_parent_action(parser, arg_base[2:])
 
         typehint = typehint_from_action(action)
-        if (
+        if typehint and (
             ActionTypeHint.is_subclass_typehint(typehint, all_subtypes=False) or
+            ActionTypeHint.is_callable_typehint(typehint, all_subtypes=False) or
             ActionTypeHint.is_mapping_typehint(typehint)
         ):
             return action, arg_base, explicit_arg
@@ -326,17 +358,10 @@ class ActionTypeHint(Action):
         if self.nargs == '?' and args[2] is None:
             val = None
         else:
-            parser, cfg, val, opt_str = args
+            cfg, val, opt_str = args[1:]
             if isinstance(opt_str, str) and opt_str.startswith(f'--{self.dest}.'):
                 sub_opt = opt_str[len(f'--{self.dest}.'):]
                 val = NestedArg(key=sub_opt, val=val)
-                if self.dest not in cfg:
-                    try:
-                        default = parser.get_default(self.dest)
-                        cfg = deepcopy(cfg)
-                        cfg[self.dest] = default
-                    except KeyError:
-                        pass
             append = opt_str == f'--{self.dest}+'
             val = self._check_type(val, append=append, cfg=cfg)
         args[1].update(val, self.dest)
@@ -355,25 +380,30 @@ class ActionTypeHint(Action):
                 except get_loader_exceptions():
                     config_path = None
                 path_meta = val.pop('__path__', None) if isinstance(val, dict) else None
+
+                prev_val = cfg.get(self.dest) if cfg else None
+                if not prev_val and not sub_defaults.get() and is_subclass_spec(self.default):
+                    prev_val = Namespace(class_path=self.default.class_path)
+
                 kwargs = {
                     'sub_add_kwargs': getattr(self, 'sub_add_kwargs', {}),
-                    'prev_val': cfg.get(self.dest) if cfg else None,
+                    'prev_val': prev_val,
                     'append': append,
                 }
                 try:
                     with change_to_path_dir(config_path):
                         val = adapt_typehints(val, self._typehint, **kwargs)
                 except ValueError as ex:
-                    val_is_int_float_or_none = isinstance(val, (int, float)) or val is None
-                    if lenient_check.get():
-                        value[num] = orig_val if val_is_int_float_or_none else val
-                        continue
-                    if val_is_int_float_or_none and config_path is None:
-                        val = adapt_typehints(orig_val, self._typehint, **kwargs)
-                    else:
+                    try:
+                        if isinstance(orig_val, str):
+                            with change_to_path_dir(config_path):
+                                val = adapt_typehints(orig_val, self._typehint, **kwargs)
+                            ex = None
+                    except ValueError:
                         if self._enable_path and config_path is None and isinstance(orig_val, str):
                             msg = f'\n- Expected a config path but "{orig_val}" either not accessible or invalid.\n- '
                             raise type(ex)(msg+str(ex)) from ex
+                    if ex:
                         raise ex
 
                 if not append and self._supports_append:
@@ -403,12 +433,15 @@ class ActionTypeHint(Action):
 
 
     @staticmethod
-    def get_class_parser(val_class, sub_add_kwargs=None):
+    def get_class_parser(val_class, sub_add_kwargs=None, skip_args=0):
         from .core import ArgumentParser
         if isinstance(val_class, str):
             val_class = import_object(val_class)
+        kwargs = dict(sub_add_kwargs) if sub_add_kwargs else {}
+        if skip_args:
+            kwargs.setdefault('skip', set()).add(skip_args)
         parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(val_class, **(sub_add_kwargs or {}))
+        parser.add_class_arguments(val_class, **kwargs)
         return parser
 
 
@@ -416,7 +449,7 @@ class ActionTypeHint(Action):
         extra = ''
         if (
             self.is_subclass_typehint(self, all_subtypes=False) or
-            get_typehint_origin(self._typehint) in {Type, type}
+            get_typehint_origin(self._typehint) in callable_origin_types.union({Type, type})
         ):
             class_paths = get_all_subclass_paths(self._typehint)
             if class_paths:
@@ -460,36 +493,44 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
         'append': append,
         'sub_add_kwargs': sub_add_kwargs or {},
     }
-    subtypehints = get_typehint_subtypes(typehint, append=append)
+    subtypehints = getattr(typehint, '__args__', None)
     typehint_origin = get_typehint_origin(typehint) or typehint
 
     # Any
     if typehint == Any:
         type_val = type(val)
-        if type_val in registered_types or is_subclass(type_val, Enum):
+        if get_registered_type(type_val) or is_subclass(type_val, Enum):
             val = adapt_typehints(val, type_val, **adapt_kwargs)
         elif isinstance(val, str):
             try:
-                val, _ = parse_value_or_config(val, enable_path=False)
+                val, _ = parse_value_or_config(val, enable_path=False, simple_types=True)
             except get_loader_exceptions():
                 pass
+        val = adapt_classes_any(val, serialize, instantiate_classes, sub_add_kwargs)
 
     # Literal
     elif typehint_origin == Literal:
+        if val not in subtypehints and isinstance(val, str):
+            subtypes = Union[tuple(set(type(v) for v in subtypehints))]
+            val = adapt_typehints(val, subtypes, **adapt_kwargs)
         if val not in subtypehints:
             raise ValueError(f'Expected a {typehint} but got "{val}"')
 
     # Basic types
     elif typehint in leaf_types:
-        if not isinstance(val, typehint):
+        if not isinstance(val, typehint) and isinstance(val, str) and typehint is not str:
+            try:
+                val = yaml_load(val)
+            except pyyaml_exceptions:
+                pass
             if typehint is float and isinstance(val, int):
                 val = float(val)
-            else:
-                raise ValueError(f'Expected a {typehint} but got "{val}"')
+        if not isinstance(val, typehint):
+            raise ValueError(f'Expected a {typehint} but got "{val}"')
 
     # Registered types
-    elif typehint in registered_types:
-        registered_type = registered_types[typehint]
+    elif get_registered_type(typehint):
+        registered_type = get_registered_type(typehint)
         if serialize:
             if registered_type.is_value_of_type(val):
                 val = registered_type.serializer(val)
@@ -523,6 +564,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
     # Union
     elif typehint_origin == Union:
         vals = []
+        subtypehints = sort_subtypes_for_append(subtypehints) if append else subtypehints
         for subtypehint in subtypehints:
             try:
                 vals.append(adapt_typehints(val, subtypehint, **adapt_kwargs))
@@ -531,7 +573,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                 vals.append(ex)
         if all(isinstance(v, Exception) for v in vals):
             e = indent_text('\n- '.join(str(v) for v in ['']+vals))
-            raise ValueError(f'Value "{val}" does not validate against any of the types in {typehint}:{e}')
+            raise ValueError(f'Value "{val}" does not validate against any of the types in {typehint}:{e}') from vals[0]
         val = [v for v in vals if not isinstance(v, Exception)][0]
 
     # Tuple or Set
@@ -545,7 +587,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             if is_tuple and not is_ellipsis and len(val) != len(subtypehints):
                 raise ValueError(f'Expected a tuple with {len(subtypehints)} elements but got "{val}"')
             for n, v in enumerate(val):
-                subtypehint = subtypehints[0 if is_ellipsis else n]
+                subtypehint = subtypehints[0 if is_ellipsis or not is_tuple else n]
                 val[n] = adapt_typehints(v, subtypehint, **adapt_kwargs)
             if is_tuple and len(val) == 0:
                 raise ValueError('Expected a non-empty tuple')
@@ -562,7 +604,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                     prev_val = [adapt_typehints(prev_val, subtypehints[0], **adapt_kwargs)]
                 except Exception:
                     prev_val = []
-            val = prev_val + (val if isinstance(val, list) else [val])
+            val = prev_val + (val if isinstance(val, list) else [adapt_typehints(val, subtypehints[0], **adapt_kwargs)])
             prev_val = prev_val + [None] * (len(val) if isinstance(val, list) else 1)
         if isinstance(val, NestedArg) and subtypehints is not None:
             val = (prev_val[:-1] if isinstance(prev_val, list) else []) + [val]
@@ -610,21 +652,24 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                 if isinstance(val, str):
                     val_obj = import_object(val)
                     if inspect.isclass(val_obj):
-                        val = {'class_path': val}
+                        val = Namespace(class_path=val)
                     elif callable(val_obj):
                         val = val_obj
                     else:
                         raise ImportError(f'Unexpected import object {val_obj}')
-                if isinstance(val, (dict, Namespace)):
+                if isinstance(val, (dict, Namespace, NestedArg)):
+                    val_input = val
+                    val = subclass_spec_as_namespace(val, prev_val)
                     if not is_subclass_spec(val):
-                        raise ImportError(f'Dict must include a class_path and optionally init_args, but got {val}')
+                        raise ImportError(f'Dict must include a class_path and optionally init_args, but got {val_input}')
+                    val, partial_classes, num_partial_args = get_partial_callable_classes(typehint, val)
                     val_class = import_object(val['class_path'])
-                    if not (inspect.isclass(val_class) and callable_instances(val_class)):
-                        raise ImportError(f'{val["class_path"]!r} is not a callable class.')
+                    if inspect.isclass(val_class) and not (partial_classes or callable_instances(val_class)):
+                        raise ImportError(f'Expected {val["class_path"]!r} to be a class that instantiates into callable or a subclass of {partial_classes}.')
                     val['class_path'] = get_import_path(val_class)
-                    val = adapt_class_type(val, False, instantiate_classes, sub_add_kwargs)
+                    val = adapt_class_type(val, False, instantiate_classes, sub_add_kwargs, skip_args=num_partial_args)
             except (ImportError, AttributeError, ParserError) as ex:
-                raise ValueError(f'Type {typehint} expects a function or a callable class: {ex}')
+                raise ValueError(f'Type {typehint} expects a function or a callable class: {ex}') from ex
 
     # Subclass
     elif not hasattr(typehint, '__origin__') and inspect.isclass(typehint):
@@ -666,6 +711,8 @@ def is_subclass_spec(val):
 
 
 def subclass_spec_as_namespace(val, prev_val=None):
+    if not isinstance(val, (str, dict, Namespace, NestedArg)):
+        return None
     if isinstance(val, str):
         return Namespace(class_path=val)
     if isinstance(val, NestedArg):
@@ -699,6 +746,26 @@ def subclass_spec_as_namespace(val, prev_val=None):
     return val
 
 
+def get_partial_callable_classes(callable_type, subclass_spec):
+    partial_classes = False
+    num_partial_args = 0
+    subtypes = getattr(callable_type, '__args__', None)
+    if subtypes and len(subtypes) > 1:
+        return_type = subtypes[-1]
+        return_origin = get_typehint_origin(return_type)
+        subclass_types = tuple(
+            t for t in (return_type.__args__ if return_origin == Union else [return_type])
+            if ActionTypeHint.is_subclass_typehint(t)
+        )
+        class_type = import_object(resolve_class_path_by_name(return_type, subclass_spec['class_path']))
+        if subclass_types and is_subclass(class_type, subclass_types):
+            subclass_spec = deepcopy(subclass_spec)
+            subclass_spec['class_path'] = get_import_path(class_type)
+            partial_classes = True
+            num_partial_args = len(subtypes)-1
+    return subclass_spec, partial_classes, num_partial_args
+
+
 def get_all_subclass_paths(cls: Type) -> List[str]:
     subclass_list = []
 
@@ -717,6 +784,11 @@ def get_all_subclass_paths(cls: Type) -> List[str]:
         for subclass in cl.__subclasses__() if hasattr(cl, '__subclasses__') else []:
             add_subclasses(subclass)
 
+    if get_typehint_origin(cls) in callable_origin_types:
+        if len(getattr(cls, '__args__', [])) < 2:
+            return subclass_list
+        cls = cls.__args__[-1]
+
     if get_typehint_origin(cls) in {Union, Type, type}:
         for arg in cls.__args__:
             if ActionTypeHint.is_subclass_typehint(arg) and arg not in {object, type}:
@@ -745,6 +817,12 @@ def resolve_class_path_by_name(cls: Type, name: str) -> str:
     return class_path
 
 
+def normalize_import_path(class_path, typehint):
+    if '.' not in class_path:
+        class_path = resolve_class_path_by_name(typehint, class_path)
+    return get_import_path(import_object(class_path))
+
+
 dump_kwargs: ContextVar = ContextVar('dump_kwargs', default={})
 
 
@@ -755,17 +833,16 @@ def dump_kwargs_context(kwargs):
 
 
 def discard_init_args_on_class_path_change(parser_or_action, prev_val, value):
-    if prev_val and 'init_args' in prev_val and prev_val['class_path'] != value.class_path:
+    if prev_val and 'init_args' in prev_val and prev_val['class_path'] != value['class_path']:
         parser = parser_or_action
         if isinstance(parser_or_action, ActionTypeHint):
             sub_add_kwargs = getattr(parser_or_action, 'sub_add_kwargs', {})
-            parser = ActionTypeHint.get_class_parser(value.class_path, sub_add_kwargs)
-        prev_val = subclass_spec_as_namespace(prev_val)
+            parser = ActionTypeHint.get_class_parser(value['class_path'], sub_add_kwargs)
         del_args = {}
         for key, val in list(prev_val.init_args.__dict__.items()):
             action = _find_action(parser, key)
             if action:
-                with lenient_check_context(lenient=False):
+                with lenient_check_context(lenient=False), load_value_context(parser.parser_mode):
                     try:
                         parser._check_value_key(action, val, key, Namespace())
                     except Exception:
@@ -774,15 +851,16 @@ def discard_init_args_on_class_path_change(parser_or_action, prev_val, value):
                 del_args[key] = prev_val.init_args.pop(key)
         if del_args:
             warnings.warn(
-                f'Due to class_path change from {prev_val.class_path!r} to {value.class_path!r}, '
-                f'discarding init_args: {del_args}.'
+                f"Due to class_path change from {prev_val['class_path']!r} to {value['class_path']!r}, "
+                f"discarding init_args: {del_args}."
             )
 
 
-def adapt_class_type(value, serialize, instantiate_classes, sub_add_kwargs, prev_val=None):
+def adapt_class_type(value, serialize, instantiate_classes, sub_add_kwargs, prev_val=None, skip_args=0):
+    prev_val = subclass_spec_as_namespace(prev_val)
     value = subclass_spec_as_namespace(value)
     val_class = import_object(value.class_path)
-    parser = ActionTypeHint.get_class_parser(val_class, sub_add_kwargs)
+    parser = ActionTypeHint.get_class_parser(val_class, sub_add_kwargs, skip_args=skip_args)
 
     # No need to re-create the linked arg but just "inform" the corresponding parser actions that it exists upstream.
     for target in sub_add_kwargs.get('linked_targets', []):
@@ -811,12 +889,18 @@ def adapt_class_type(value, serialize, instantiate_classes, sub_add_kwargs, prev
             if init_args:
                 value['init_args'] = init_args
             return value
+        if skip_args:
+            def partial_instance(*args):
+                return val_class(*args, **{**init_args, **dict_kwargs})
+            return partial_instance
         return val_class(**{**init_args, **dict_kwargs})
 
+    prev_init_args = prev_val.get('init_args') if isinstance(prev_val, Namespace) else None
+
     if isinstance(init_args, NestedArg):
         value['init_args'] = parser.parse_args(
             [f'--{init_args.key}={init_args.val}'],
-            namespace=prev_val.init_args.clone(),
+            namespace=prev_init_args,
             defaults=sub_defaults.get(),
         )
         return value
@@ -832,16 +916,41 @@ def adapt_class_type(value, serialize, instantiate_classes, sub_add_kwargs, prev
         elif dict_kwargs:
             init_args['dict_kwargs'] = dict_kwargs
             dict_kwargs = None
-        init_args = parser.parse_object(init_args, defaults=sub_defaults.get())
+        init_args = parser.parse_object(init_args, cfg_base=prev_init_args, defaults=sub_defaults.get())
         if init_args:
             value['init_args'] = init_args
     if dict_kwargs:
-        if isinstance(prev_val, Namespace) and prev_val.get('class_path') == value['class_path'] and prev_val.get('dict_kwargs'):
-            dict_kwargs.update(prev_val.get('dict_kwargs'))
-        value['dict_kwargs'] = dict_kwargs
+        if prev_val and prev_val.get('class_path') == value['class_path'] and prev_val.get('dict_kwargs'):
+            dict_kwargs = {**prev_val.get('dict_kwargs'), **dict_kwargs}
+        value['dict_kwargs'] = {
+            k: load_value(v, simple_types=True) if isinstance(v, str) else v
+            for k, v in dict_kwargs.items()
+        }
     return value
 
 
+def adapt_classes_any(val, serialize, instantiate_classes, sub_add_kwargs):
+    if is_subclass_spec(val):
+        orig_val = val
+        val = subclass_spec_as_namespace(val)
+        init_args = val.get('init_args')
+        if init_args and not instantiate_classes:
+            for subkey, subval in init_args.__dict__.items():
+                init_args[subkey] = adapt_classes_any(subval, serialize, instantiate_classes, sub_add_kwargs)
+            val['init_args'] = init_args
+        try:
+            val = adapt_class_type(val, serialize, instantiate_classes, sub_add_kwargs)
+        except Exception:
+            return orig_val
+    elif isinstance(val, list):
+        for num, subval in enumerate(val):
+            val[num] = adapt_classes_any(subval, serialize, instantiate_classes, sub_add_kwargs)
+    elif isinstance(val, dict):
+        for key, subval in val.items():
+            val[key] = adapt_classes_any(subval, serialize, instantiate_classes, sub_add_kwargs)
+    return val
+
+
 def not_append_diff(val1, val2):
     if isinstance(val1, list) and isinstance(val2, list):
         val1 = [x.get('class_path') if is_subclass_spec(x) else x for x in val1]
@@ -849,9 +958,8 @@ def not_append_diff(val1, val2):
     return val1 != val2
 
 
-def get_typehint_subtypes(typehint, append):
-    subtypes = getattr(typehint, '__args__', None)
-    if append and subtypes:
+def sort_subtypes_for_append(subtypes):
+    if subtypes and len(subtypes) > 1:
         subtypes = sorted(subtypes, key=lambda x: get_typehint_origin(x) not in sequence_origin_types)
     return subtypes
 
@@ -959,7 +1067,6 @@ class LazyInitBaseClass:
     def __init__(self, class_type: Type, lazy_kwargs: dict):
         check_lazy_kwargs(class_type, lazy_kwargs)
         self._lazy_class_type = class_type
-        self._lazy_class_path = get_import_path(class_type)
         self._lazy_kwargs = lazy_kwargs
         self._lazy_methods = {}
         seen_methods: Dict = {}
@@ -989,7 +1096,7 @@ class LazyInitBaseClass:
         init_args = Namespace(self._lazy_kwargs)
         if is_final_class(self._lazy_class_type):
             return init_args
-        init = Namespace(class_path=self._lazy_class_path)
+        init = Namespace(class_path=get_import_path(self._lazy_class_type))
         if len(self._lazy_kwargs) > 0:
             init['init_args'] = init_args
         return init
