@@ -32,7 +32,6 @@ from jsonargparse import (
 from jsonargparse.namespace import meta_keys
 from jsonargparse.optionals import (
     docstring_parser_support,
-    dump_preserve_order_support,
     fsspec_support,
     jsonnet_support,
     jsonschema_support,
@@ -41,7 +40,7 @@ from jsonargparse.optionals import (
 )
 from jsonargparse.typing import NotEmptyStr, Path_fc, Path_fr, PositiveFloat, PositiveInt
 from jsonargparse.util import CaptureParserException, capture_parser, DebugException, null_logger
-from jsonargparse_tests.base import is_posix, responses_activate, responses_available, TempDirTestCase
+from jsonargparse_tests.base import is_cpython, is_posix, responses_activate, responses_available, TempDirTestCase
 
 
 def example_parser():
@@ -680,6 +679,7 @@ class OutputTests(TempDirTestCase):
         self.assertEqual(parser.dump(cfg), 'paths:\n- path1\n- path2\n')
 
 
+    @unittest.skipIf(not is_cpython, 'requires __setattr__ insertion order')
     def test_dump_formats(self):
         parser = ArgumentParser()
         parser.add_argument('--op1', default=123)
@@ -711,8 +711,6 @@ class OutputTests(TempDirTestCase):
         self.assertEqual(parser.dump(parser.parse_args(['--g2.op2=pqr']), skip_default=True), 'g2:\n  op2: pqr\n')
 
 
-    @unittest.skipIf(not dump_preserve_order_support,
-                     'Dump preserve order only supported in python>=3.6 and CPython')
     def test_dump_order(self):
         args = {}
         for num in range(50):
@@ -1153,6 +1151,16 @@ class OtherTests(unittest.TestCase):
         parser = ArgumentParser()
         self.assertRaises(NotImplementedError, lambda: parser.parse_known_args([]))
 
+    def test_parse_known_args_without_caller_module(self):
+        """
+        Test case for corner case when calling parse_known_args in IPython.
+        The caller module will not exist. Test this is handled.
+        See https://github.com/omni-us/jsonargparse/pull/179
+        """
+        with unittest.mock.patch('inspect.getmodule') as mock_getmodule:
+            mock_getmodule.return_value = None
+            parser = ArgumentParser()
+            self.assertRaises(NotImplementedError, lambda: parser.parse_known_args([]))
 
     def test_parse_args_invalid_args(self):
         parser = ArgumentParser(error_handler=None)
@@ -1188,5 +1196,19 @@ class OtherTests(unittest.TestCase):
             capture_parser(lambda: None)
 
 
+    def test_set_defaults_config_argument_error(self):
+        parser = ArgumentParser()
+        parser.add_argument('--config', action=ActionConfigFile)
+        with self.assertRaises(ValueError):
+            parser.set_defaults(config='config.yaml')
+
+
+    def test_add_multiple_config_arguments_error(self):
+        parser = ArgumentParser()
+        parser.add_argument('--cfg1', action=ActionConfigFile)
+        with self.assertRaises(ValueError):
+            parser.add_argument('--cfg2', action=ActionConfigFile)
+
+
 if __name__ == '__main__':
     unittest.main(verbosity=2)
