@@ -121,7 +121,6 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual('a', Class3(**cfg.as_dict())())
 
         self.assertRaises(ParserError, lambda: parser.parse_args([]))  # c3_a0 is required
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--c3_a0=0', '--c3_a7=["3", "3", 3.0]']))  # tuple[1] is int
 
         if docstring_parser_support:
             self.assertEqual('Class3 short description', parser.groups['Class3'].title)
@@ -152,6 +151,29 @@ class SignaturesTests(unittest.TestCase):
         self.assertRaises(ValueError, lambda: parser.add_class_arguments(Class2))
 
 
+    def test_add_class_without_args(self):
+        class NoArgs:
+            pass
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--cfg', action=ActionConfigFile)
+        parser.add_class_arguments(NoArgs, 'noargs')
+
+        help_str = StringIO()
+        parser.print_help(help_str)
+        self.assertNotIn('noargs', help_str.getvalue())
+
+        cfg = parser.parse_args([])
+        self.assertNotIn('noargs', cfg)
+        init = parser.instantiate_classes(cfg)
+        self.assertIsInstance(init.noargs, NoArgs)
+
+        with mock_module(NoArgs) as module:
+            config = {'noargs': {'class_path': f'{module}.NoArgs'}}
+            with self.assertRaises(ParserError):
+                parser.parse_args([f'--cfg={config}'])
+
+
     def test_add_class_without_valid_args(self):
         class NoValidArgs:
             def __init__(self, a0=None):
@@ -425,10 +447,9 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, cal: Union[Calendar, bool] = lazy_instance(MyCalendar, param=1)):
                 self.cal = cal
 
-        parser = ArgumentParser(error_handler=None)
-        parser.add_class_arguments(Main, 'main')
-
-        with warnings.catch_warnings(record=True) as w:
+        with mock_module(MyCalendar), warnings.catch_warnings(record=True) as w:
+            parser = ArgumentParser(error_handler=None)
+            parser.add_class_arguments(Main, 'main')
             parser.parse_args(['--main.cal=Calendar'])
             self.assertIn("discarding init_args: {'param': 1}", str(w[0].message))
 
@@ -609,13 +630,20 @@ class SignaturesTests(unittest.TestCase):
                  a4: int = 4):
             return a1
 
-        parser = ArgumentParser()
-        parser.add_function_arguments(func, skip={'a2', 'a4'})
+        with self.subTest('parameter names'):
+            parser = ArgumentParser()
+            added_args = parser.add_function_arguments(func, skip={'a2', 'a4'})
+            self.assertEqual(added_args, ['a1', 'a3'])
 
-        for key in ['a1', 'a3']:
-            self.assertIsNotNone(_find_action(parser, key), key+' should be in parser but is not')
-        for key in ['a2', 'a4']:
-            self.assertIsNone(_find_action(parser, key), key+' should not be in parser but is')
+        with self.subTest('positionals and names'):
+            parser = ArgumentParser()
+            added_args = parser.add_function_arguments(func, skip={1, 'a3'})
+            self.assertEqual(added_args, ['a2', 'a4'])
+
+        with self.subTest('invalid skip'):
+            parser = ArgumentParser()
+            with self.assertRaises(ValueError):
+                parser.add_function_arguments(func, skip={1, 2})
 
 
     def test_skip_within_subclass_type(self):
