@@ -6,22 +6,24 @@ import pathlib
 import random
 import re
 import sys
+import time
 import unittest
 import uuid
 import warnings
 import yaml
-from calendar import Calendar
+from calendar import Calendar, HTMLCalendar, TextCalendar
 from contextlib import redirect_stderr, redirect_stdout
 from copy import deepcopy
 from datetime import datetime
 from enum import Enum
 from gzip import GzipFile
 from io import StringIO
-from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, Type, Union
+from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Set, Tuple, Type, Union
 from jsonargparse import ActionConfigFile, ArgumentParser, CLI, lazy_instance, Namespace, ParserError, Path
 from jsonargparse.typehints import ActionTypeHint, is_optional, Literal
 from jsonargparse.typing import (
     Email,
+    final,
     NotEmptyStr,
     OpenUnitInterval,
     Path_drw,
@@ -87,6 +89,7 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual('  ', parser.parse_args(['--op', '  ']).op)
         self.assertEqual('', parser.parse_args(['--op', '']).op)
         self.assertEqual(' abc ', parser.parse_args(['--op= abc ']).op)
+        self.assertEqual('xyz: ', parser.parse_args(['--op=xyz: ']).op)
         self.assertEqual(' ', parser.parse_args(['--cfg={"op":" "}']).op)
         self.assertIsNone(parser.parse_args(['--op=null']).op)
 
@@ -98,6 +101,28 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual('2022-04-32', parser.parse_args(['2022-04-32']).foo)
 
 
+    def test_float_scientific_notation(self):
+        parser = ArgumentParser()
+        parser.add_argument('--num', type=float)
+        self.assertEqual(1e-3, parser.parse_args(['--num=1e-3']).num)
+
+
+    def test_str_with_number_value(self):
+        class Class:
+            def __init__(self, val: str = '-'):
+                pass
+
+        with mock_module(Class):
+            parser = ArgumentParser()
+            parser.add_argument('--val', type=str)
+            parser.add_argument('--cls', type=Class, default=lazy_instance(Class))
+
+            for value in ['1', '02', '3.40', '5.7e-8']:
+                with self.subTest(value):
+                    self.assertEqual(value, parser.parse_args([f'--val={value}']).val)
+                    self.assertEqual(value, parser.parse_args([f'--cls.val={value}']).cls.init_args.val)
+
+
     def test_list(self):
         for list_type in [Iterable, List, Sequence]:
             with self.subTest(str(list_type)):
@@ -183,6 +208,13 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual({'dict1': {'2': 4.5, '6': 'ab'}, 'dict2': {'a': True, 'b': 'f'}}, cfg)
 
 
+    def test_set(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--set', type=Set[int])
+        self.assertEqual({1, 2}, parser.parse_args(['--set=[1, 2]']).set)
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--set=["a", "b"]']))
+
+
     def test_tuple(self):
         class MyEnum(Enum):
             ab = 1
@@ -195,7 +227,6 @@ class TypeHintsTests(unittest.TestCase):
         self.assertRaises(ParserError, lambda: parser.parse_args(['--tuple=[]']))
         self.assertRaises(ParserError, lambda: parser.parse_args(['--tuple=[2, "a", "b", 5]']))
         self.assertRaises(ParserError, lambda: parser.parse_args(['--tuple=[2, "a"]']))
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--tuple=["2", "a", "b"]']))
         self.assertRaises(ParserError, lambda: parser.parse_args(['--tuple={"a":1, "b":"2"}']))
         out = StringIO()
         parser.print_help(out)
@@ -278,6 +309,40 @@ class TypeHintsTests(unittest.TestCase):
         self.assertRaises(ParserError, lambda: parser.parse_args(['--cfg', 'val+: a']))
 
 
+    def test_list_append_subclass_init_args(self):
+        class Class:
+            def __init__(self, p1: int = 0, p2: int = 0):
+                pass
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--val', type=Union[Class, List[Class]])
+
+        with mock_module(Class) as module:
+            cfg = parser.parse_args([f'--val+={module}.Class', '--val.p1=1', '--val.p2=2', '--val.p1=3'])
+            self.assertEqual(cfg.val, [Namespace(class_path=f'{module}.Class', init_args=Namespace(p1=3, p2=2))])
+            cfg = parser.parse_args([f'--val+=Class', '--val.p2=2', '--val.p1=1'])
+            self.assertEqual(cfg.val, [Namespace(class_path=f'{module}.Class', init_args=Namespace(p1=1, p2=2))])
+
+
+    def test_list_append_subclass_nonclass_default(self):
+        @final
+        class Class:
+            def __init__(self, cal: Union[Calendar, Iterable[Calendar], bool] = True):
+                self.cal = cal
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('cls', type=Class)
+
+        cfg = parser.parse_args(['--cls.cal=calendar.TextCalendar', '--cls.cal.firstweekday=2'])
+        self.assertNotIsInstance(cfg.cls.cal, list)
+        self.assertEqual('calendar.TextCalendar', cfg.cls.cal.class_path)
+        self.assertEqual(2, cfg.cls.cal.init_args.firstweekday)
+
+        cfg = parser.parse_args(['--cls.cal=calendar.TextCalendar', '--cls.cal+=calendar.HTMLCalendar'])
+        self.assertIsInstance(cfg.cls.cal, list)
+        self.assertEqual(['TextCalendar', 'HTMLCalendar'], [c.class_path.split('.')[1] for c in cfg.cls.cal])
+
+
     def test_list_append_subcommand_subclass(self):
         class A:
             def __init__(self, cals: Union[Calendar, List[Calendar]] = None):
@@ -332,6 +397,125 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual('[[[', parser.parse_args(['--any=[[['])['any'])
 
 
+    def test_type_any_subclasses(self):
+        class Class:
+            def __init__(self, cal1: Calendar, cal2: Any):
+                self.cal1 = cal1
+                self.cal2 = cal2
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--any', type=Any)
+
+        with mock_module(Class) as module:
+            value = {
+                'class_path': f'{module}.Class',
+                'init_args': {
+                    'cal1': {
+                        'class_path': 'calendar.TextCalendar',
+                        'init_args': {'firstweekday': 1},
+                    },
+                    'cal2': {
+                        'class_path': 'calendar.HTMLCalendar',
+                        'init_args': {'firstweekday': 2},
+                    },
+                },
+            }
+
+            cfg = parser.parse_args([f'--any={value}'])
+            init = parser.instantiate_classes(cfg)
+            self.assertIsInstance(init.any, Class)
+            self.assertIsInstance(init.any.cal1, TextCalendar)
+            self.assertIsInstance(init.any.cal2, HTMLCalendar)
+            self.assertEqual(init.any.cal1.firstweekday, 1)
+            self.assertEqual(init.any.cal2.firstweekday, 2)
+
+            value['init_args']['cal2']['class_path'] = 'does.not.exist'
+            cfg = parser.parse_args([f'--any={value}'])
+            self.assertIsInstance(cfg.any.init_args.cal1, Namespace)
+            self.assertIsInstance(cfg.any.init_args.cal2, dict)
+            init = parser.instantiate_classes(cfg)
+            self.assertIsInstance(init.any, Class)
+            self.assertIsInstance(init.any.cal1, TextCalendar)
+            self.assertIsInstance(init.any.cal2, dict)
+            self.assertEqual(init.any.cal1.firstweekday, 1)
+            self.assertEqual(init.any.cal2['init_args']['firstweekday'], 2)
+
+            value['init_args']['cal1']['class_path'] = 'does.not.exist'
+            cfg = parser.parse_args([f'--any={value}'])
+            self.assertIsInstance(cfg.any, dict)
+
+
+    def test_type_any_list_of_subclasses(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--any', type=Any)
+
+        value = [
+            {
+                'class_path': 'calendar.TextCalendar',
+                'init_args': {'firstweekday': 1},
+            },
+            {
+                'class_path': 'calendar.HTMLCalendar',
+                'init_args': {'firstweekday': 2},
+            },
+        ]
+
+        cfg = parser.parse_args([f'--any={value}'])
+        init = parser.instantiate_classes(cfg)
+        self.assertIsInstance(init.any, list)
+        self.assertEqual(len(init.any), 2)
+        self.assertIsInstance(init.any[0], TextCalendar)
+        self.assertIsInstance(init.any[1], HTMLCalendar)
+        self.assertEqual(init.any[0].firstweekday, 1)
+        self.assertEqual(init.any[1].firstweekday, 2)
+
+
+    def test_type_any_dict_of_subclasses(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--any', type=Any)
+
+        value = {
+            'k1': {
+                'class_path': 'calendar.TextCalendar',
+                'init_args': {'firstweekday': 1},
+            },
+            'k2': {
+                'class_path': 'calendar.HTMLCalendar',
+                'init_args': {'firstweekday': 2},
+            },
+        }
+
+        cfg = parser.parse_args([f'--any={value}'])
+        init = parser.instantiate_classes(cfg)
+        self.assertIsInstance(init.any, dict)
+        self.assertEqual(len(init.any), 2)
+        self.assertIsInstance(init.any['k1'], TextCalendar)
+        self.assertIsInstance(init.any['k2'], HTMLCalendar)
+        self.assertEqual(init.any['k1'].firstweekday, 1)
+        self.assertEqual(init.any['k2'].firstweekday, 2)
+
+
+    def test_union_subtypes_order(self):
+        for subtypes, arg, expected in [
+            ((bool, str), '=true', True),
+            ((str, bool), '=true', 'true'),
+            ((int, str), '=1', 1),
+            ((str, int), '=2', '2'),
+            ((float, int), '=3', 3.0),
+            ((int, float), '=4', 4),
+            ((int, List[int]), '=5', 5),
+            ((List[int], int), '=6', 6),
+            ((int, List[int]), '+=7', [7]),
+            ((List[int], int), '+=8', [8]),
+        ]:
+            with self.subTest(f'{subtypes}, {arg}, {expected}'):
+                parser = ArgumentParser()
+                parser.add_argument('--val', type=Union[subtypes])
+                val = parser.parse_args([f'--val{arg}']).val
+                self.assertIsInstance(val, type(expected))
+                self.assertEqual(val, expected)
+
+
     @unittest.skipIf(not Literal, 'Literal introduced in python 3.8 or backported in typing_extensions')
     def test_Literal(self):
         parser = ArgumentParser(error_handler=None)
@@ -418,9 +602,14 @@ class TypeHintsTests(unittest.TestCase):
 
     def test_Callable_with_function_path(self):
         parser = ArgumentParser(error_handler=None)
-        parser.add_argument('--callable', type=Callable, default=lazy_instance)
+        parser.add_argument('--callable', type=Callable, default=time.time)
         parser.add_argument('--list', type=List[Callable])
 
+        cfg = parser.get_defaults()
+        self.assertEqual(time.time, cfg.callable)
+        self.assertEqual(parser.dump(cfg), 'callable: time.time\n')
+        cfg = parser.parse_args(['--callable=random.randint'])
+        self.assertEqual(random.randint, cfg.callable)
         cfg = parser.parse_args(['--callable=jsonargparse.CLI'])
         self.assertEqual(CLI, cfg.callable)
         self.assertEqual(parser.dump(cfg), 'callable: jsonargparse.CLI\n')
@@ -429,7 +618,7 @@ class TypeHintsTests(unittest.TestCase):
 
         out = StringIO()
         parser.print_help(out)
-        self.assertIn('(type: Callable, default: jsonargparse.lazy_instance)', out.getvalue())
+        self.assertIn('(type: Callable, default: time.time)', out.getvalue())
 
 
     def test_Callable_with_class_path(self):
@@ -464,6 +653,42 @@ class TypeHintsTests(unittest.TestCase):
             self.assertRaises(ParserError, lambda: parser.parse_args([f'--callable={json.dumps(value)}']))
 
 
+    def test_callable_with_class_path_short_init_args(self):
+        class MyCallable:
+            def __init__(self, name: str):
+                self.name = name
+            def __call__(self):
+                return self.name
+
+        parser = ArgumentParser()
+        parser.add_argument('--call', type=Callable)
+
+        with mock_module(MyCallable) as module:
+            cfg = parser.parse_args([f'--call={module}.MyCallable', '--call.name=Bob'])
+            self.assertEqual(cfg.call.class_path, f'{module}.MyCallable')
+            self.assertEqual(cfg.call.init_args, Namespace(name='Bob'))
+            init = parser.instantiate_classes(cfg)
+            self.assertEqual(init.call(), 'Bob')
+
+
+    def test_union_callable_with_class_path_short_init_args(self):
+        class MyCallable:
+            def __init__(self, name: str):
+                self.name = name
+            def __call__(self):
+                return self.name
+
+        parser = ArgumentParser()
+        parser.add_argument('--call', type=Union[Callable, None])
+
+        with mock_module(MyCallable) as module:
+            cfg = parser.parse_args([f'--call={module}.MyCallable', '--call.name=Bob'])
+            self.assertEqual(cfg.call.class_path, f'{module}.MyCallable')
+            self.assertEqual(cfg.call.init_args, Namespace(name='Bob'))
+            init = parser.instantiate_classes(cfg)
+            self.assertEqual(init.call(), 'Bob')
+
+
     def test_typed_Callable_with_function_path(self):
         def my_func_1(p: int) -> str:
             return str(p)
@@ -481,6 +706,101 @@ class TypeHintsTests(unittest.TestCase):
             self.assertEqual(my_func_2, cfg.callable)  # Currently callable types are ignored
 
 
+    def test_typed_callable_with_return_type_class(self):
+        class Optimizer:
+            def __init__(self, params: List[float], lr: float = 1e-3):
+                self.params = params
+                self.lr = lr
+
+        class SGD(Optimizer):
+            def __init__(self, *args, **kwargs):
+                super().__init__(*args, **kwargs)
+
+        class Adam(Optimizer):
+            def __init__(self, *args, **kwargs):
+                super().__init__(*args, **kwargs)
+
+        value = {
+            'class_path': 'Adam',
+            'init_args': {
+                'lr': 0.01,
+            }
+        }
+
+        with mock_module(Optimizer, SGD, Adam) as module:
+            parser = ArgumentParser(error_handler=None)
+            parser.add_argument('--optimizer', type=Callable[[List[float]], Optimizer], default=SGD)
+
+            cfg = parser.get_defaults()
+            init = parser.instantiate_classes(cfg)
+            optim = init.optimizer([0.1, 2,3])
+            self.assertIsInstance(optim, SGD)
+            self.assertEqual(optim.params, [0.1, 2,3])
+            self.assertEqual(optim.lr, 1e-3)
+
+            cfg = parser.parse_args(['--optimizer', str(value)])
+            self.assertEqual(cfg.optimizer.class_path, f'{module}.Adam')
+            self.assertEqual(cfg.optimizer.init_args, Namespace(lr=0.01))
+            init = parser.instantiate_classes(cfg)
+            optim = init.optimizer([4.5, 6.7])
+            self.assertIsInstance(optim, Adam)
+            self.assertEqual(optim.params, [4.5, 6.7])
+            self.assertEqual(optim.lr, 0.01)
+
+            help_str = StringIO()
+            parser.print_help(help_str)
+            for name in ['Optimizer', 'SGD', 'Adam']:
+                self.assertIn(f'{module}.{name}', help_str.getvalue())
+
+
+    def test_typed_callable_with_return_type_union_of_classes(self):
+        class Optimizer:
+            pass
+
+        class StepLR:
+            def __init__(self, optimizer: Optimizer, last_epoch: int = -1):
+                self.optimizer = optimizer
+                self.last_epoch = last_epoch
+
+        class ReduceLROnPlateau:
+            def __init__(self, optimizer: Optimizer, monitor: str):
+                self.optimizer = optimizer
+                self.monitor = monitor
+
+        optim = Optimizer()
+        value = {
+            'class_path': 'ReduceLROnPlateau',
+            'init_args': {
+                'monitor': 'loss',
+            }
+        }
+
+        with mock_module(StepLR, ReduceLROnPlateau) as module:
+            parser = ArgumentParser(error_handler=None)
+            parser.add_argument('--scheduler', type=Callable[[Optimizer], Union[StepLR, ReduceLROnPlateau]], default=StepLR)
+
+            cfg = parser.get_defaults()
+            init = parser.instantiate_classes(cfg)
+            sched = init.scheduler(optim)
+            self.assertIsInstance(sched, StepLR)
+            self.assertIs(sched.optimizer, optim)
+            self.assertEqual(sched.last_epoch, -1)
+
+            cfg = parser.parse_args(['--scheduler', str(value)])
+            self.assertEqual(cfg.scheduler.class_path, f'{module}.ReduceLROnPlateau')
+            self.assertEqual(cfg.scheduler.init_args, Namespace(monitor='loss'))
+            init = parser.instantiate_classes(cfg)
+            sched = init.scheduler(optim)
+            self.assertIsInstance(sched, ReduceLROnPlateau)
+            self.assertIs(sched.optimizer, optim)
+            self.assertEqual(sched.monitor, 'loss')
+
+            help_str = StringIO()
+            parser.print_help(help_str)
+            for name in ['StepLR', 'ReduceLROnPlateau']:
+                self.assertIn(f'{module}.{name}', help_str.getvalue())
+
+
     def test_class_type(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--op', type=Optional[List[Calendar]])
@@ -588,6 +908,21 @@ class TypeHintsTests(unittest.TestCase):
             self.assertIn(f'{module}.HTMLCalendar', err.getvalue())
 
 
+    def test_class_type_set_defaults_class_name(self):
+        parser = ArgumentParser()
+        parser.add_argument('--cal', type=Calendar)
+        parser.set_defaults({
+            'cal': {
+                'class_path': 'TextCalendar',
+                'init_args': {
+                    'firstweekday': 1,
+                }
+            }
+        })
+        cal = parser.get_default('cal').as_dict()
+        self.assertEqual(cal, {'class_path': 'calendar.TextCalendar', 'init_args': {'firstweekday': 1}})
+
+
     def test_class_type_subclass_short_init_args(self):
         parser = ArgumentParser()
         parser.add_argument('--op', type=Calendar)
@@ -708,6 +1043,21 @@ class TypeHintsTests(unittest.TestCase):
             self.assertEqual(cfg.data.init_args, Namespace(p1=2, p2='y', p3=True))
 
 
+    def test_class_type_subclass_in_union_help(self):
+        parser = ArgumentParser()
+        parser.add_argument('--op', type=Union[str, Mapping[str, int], Calendar])
+
+        out = StringIO()
+        with redirect_stdout(out), self.assertRaises(SystemExit):
+            parser.parse_args(['--help'])
+        self.assertIn('Show the help for the given subclass of Calendar', out.getvalue())
+
+        out = StringIO()
+        with redirect_stdout(out), self.assertRaises(SystemExit):
+            parser.parse_args([f'--op.help=TextCalendar'])
+        self.assertIn('--op.init_args.firstweekday', out.getvalue())
+
+
     def test_class_type_subclass_nested_help(self):
         class Class:
             def __init__(self, cal: Calendar, p1: int = 0):
@@ -762,7 +1112,7 @@ class TypeHintsTests(unittest.TestCase):
             self.assertIsInstance(cfg_init.cls, Class)
             self.assertEqual(cfg_init.cls.kwargs, expected.dict_kwargs)
 
-            cfg = parser.parse_args(['--cls=Class', '--cls.dict_kwargs.p4=x', '--cls.dict_kwargs.p3=7.0'])
+            cfg = parser.parse_args(['--cls=Class', '--cls.dict_kwargs.p4=-', '--cls.dict_kwargs.p3=7.0', '--cls.dict_kwargs.p4=x'])
             self.assertEqual(cfg.cls.dict_kwargs, expected.dict_kwargs)
 
             with self.assertRaises(ParserError):
@@ -819,6 +1169,19 @@ class TypeHintsTests(unittest.TestCase):
         self.assertIn('known subclasses: calendar.Calendar,', out.getvalue())
 
 
+    def test_class_type_required(self):
+        parser = ArgumentParser()
+        parser.add_argument('cal', type=Calendar)
+
+        cfg = parser.parse_args(['TextCalendar'])
+        self.assertEqual(cfg.cal.class_path, 'calendar.TextCalendar')
+
+        help_str = StringIO()
+        parser.print_help(help_str)
+        self.assertIn("required, type: <class 'Calendar'>", help_str.getvalue())
+        self.assertIn('--cal.help', help_str.getvalue())
+
+
     def test_typehint_serialize_list(self):
         parser = ArgumentParser()
         action = parser.add_argument('--list', type=Union[PositiveInt, List[PositiveInt]])
@@ -839,8 +1202,24 @@ class TypeHintsTests(unittest.TestCase):
 
 
     def test_unsupported_type(self):
-        self.assertRaises(ValueError, lambda: ActionTypeHint(typehint=lambda: None))
-        self.assertRaises(ValueError, lambda: ActionTypeHint(typehint=Union[int, lambda: None]))
+        for typehint in [
+            lambda: None,
+            'unsupported',
+            Optional['unsupported'],
+            Tuple[int, 'unsupported'],
+            Union['unsupported1', 'unsupported2'],
+        ]:
+            with self.subTest(typehint):
+                with self.assertRaises(ValueError):
+                    ActionTypeHint(typehint=typehint)
+
+
+    def test_union_partially_unsupported_type(self):
+        parser = ArgumentParser(logger={'level': 'DEBUG'})
+        with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
+            parser.add_argument('--union', type=Union[int, str, 'unsupported'])
+            self.assertEqual(1, len(log.output))
+            self.assertIn('Discarding unsupported subtypes', log.output[0])
 
 
     def test_nargs_questionmark(self):
@@ -1084,6 +1463,32 @@ class TypeHintsTmpdirTests(TempDirTestCase):
         self.assertEqual(cfg.data.cal, Namespace(class_path='calendar.Calendar'))
 
 
+    def test_class_path_override_config_with_defaults(self):
+        class Base:
+            def __init__(self, b: int = 1):
+                pass
+
+        class Subclass1(Base):
+            def __init__(self, s1: str = '-'):
+                pass
+
+        class Subclass2(Base):
+            def __init__(self, s2: str = '-'):
+                pass
+
+        with mock_module(Base, Subclass1, Subclass2) as module:
+            parser = ArgumentParser()
+            parser.add_argument('--cfg', action=ActionConfigFile)
+            parser.add_argument('--s', type=Base, default=lazy_instance(Subclass1, s1='v1'))
+
+            config = {'s': {'class_path': 'Subclass2', 'init_args': {'s2': 'v2'}}}
+            with warnings.catch_warnings(record=True) as w:
+                cfg = parser.parse_args([f'--cfg={config}'])
+                self.assertIn("discarding init_args: {'s1': 'v1'}", str(w[0].message))
+            self.assertEqual(cfg.s.class_path, f'{module}.Subclass2')
+            self.assertEqual(cfg.s.init_args, Namespace(s2='v2'))
+
+
     def test_class_path_override_with_default_config_files(self):
 
         class MyCalendar(Calendar):
